/*

	demo.c

		8-bit ATARI computer graphics library test program and demo.
		
	author:
	
	Bart Jaszcz

*/

//#include <stdio.h>
#include <stdlib.h>
//#include <string.h>

#include "trig.h"
#include "graphics.h"

// POKEY chip random number generator
#define RAND ((byte*)0xd20a)
// disable OS keyboard handler flag location
#define KEYDIS ((byte*)0x26d)
// keyboard status
#define KBREST ((byte*) 0xd20a)              // reset keyboard
#define SKSTAT ((byte*) 0xd20f)              // keyboard status register
// keyboard flags
#define KB_OVERRUN (1<<5)
#define KB_PRESSED (1<<2)
// system time (24 bit) in vblanks
#define STIME ((byte*) 0x12)
// location that determines the vblank value (50/60 Hz)
#define COLPM2 ((byte*) 0xd014)
// fractal interations
#define FRACT 255
// real-time clock timer copy
byte time[3];
// vertex array (counter-clockwise ordering)
byte vertex[20];
// program color palette
byte palette0[4] = { 0x12, 0x25, 0x48, 0x6b }; 
byte palette1[4] = { 0x03, 0x06, 0x79, 0xfa }; 
byte palette2[4] = { 0x04, 0x07, 0x97, 0xcf }; 
byte palette3[4] = { 0x03, 0x81, 0xd4, 0xe6 }; 
byte palette4[4] = { 0x03, 0x06, 0x09, 0x0c }; 
// get center of screen in fixed point
int cx = ((WIDTH/2)<<FP_SHIFT);
int cy = ((HEIGHT/2)<<FP_SHIFT);
// major/minor radius for shapes
byte rx = 50;
byte ry = 80;
// random vertex indexes for fractal
byte rand_index[3][FRACT];
// time functions
//ulong get_time(void);				// get current time in ticks (TV vblanks)
// shape functions
void create_shape(byte, int);		// create basic shape (triangle, square, etc..)
// fractal functions
void init_fractal(byte);				// initialize fractal optimization tables
void draw_fractal(byte);				// draw static fractal
void quad_fractal(byte);				// draw dynamic fractal (quad only)
void draw_fade(void);					// draw fade fx
void full_rect(void);					// make shape for entire screen
extern void fractal(void);				// assembler fractal  (static)
extern void quad_rand();				// assembler fractal for quads (dynamic)
//void wait_for_key(void);

// start of program
int main(void)
{
	// timer
	//	ulong t0, t1;  // time, end time
	// indexes
	byte i, j, k;
	// 8 bit coordinates
	byte x, y, x1, y1;
	// line draw step
	byte step;
	// screen buffer
	byte *buffer;
	// 16 bit values
	int angle;
	// puts("\n\n  starting...\n\n");
	// init fractals for shapes
	for (i = 3; i < 6; i++)			// from triangle to pentagon
	init_fractal(i);
	
	// disable OS keyboard handler
	//	*KEYDIS = 0xff;

	// allocate double buffered screen memory
	buffer = malloc(2*BUF+3000);	// 3000 extra bytes for 4K boundry alignment
	// start timer
	//	t0 = get_time();

	// set up single buffered mode
	// puts("  init graphics..\n");
	init_graphics(2, buffer);
	set_palette(palette4);
	toggle_screen();
	// draw fractals
	set_color(1);
	clear_screen();
	j = 0;
	for (k = 0; k < 25; k++)
	{
		for (i = 5; i >= 3; i--)		// go from pentagon to triangle
		{
			set_color(++j & 0x3);			// set next color
			create_shape(i, 256 / i);
			draw_fractal(i);
			set_color(2);
			print_text(22,12,"Fake Rotation");
			print_text(22,13,"by Bart Jaszcz");
			set_color(1);
		}
	}
	// draw by quad fractal
	set_color(1);
	clear_screen();
	full_rect();
	for (i = 0; i < 40; i++)
	{
		set_color(i & 0x3);
		quad_fractal(0);
	}
	
	// draw diagonal lines
	for (i = 0; i < 5; i++) 
	{
		for (step = 79 - (i << 2); step > 3; step /= 3)
		{
			x = 0;
			for (y = 0; y < HEIGHT; y += step)
			{
				set_color(++x & 0x3);
				draw_line(0, y, WIDTH - 1, HEIGHT - y);
			}
			y = 0;
			for (x = 0; x < WIDTH; x += step)
			{
				set_color(++y & 0x3);
				draw_line(x, 0, WIDTH - x, HEIGHT - 1);
			}
		}
	}
	// erase by quad fractal
	set_palette(palette3);
	set_color(1);
	i = 0;									// 0 == 256 interations
	while(--i)
	quad_fractal(0);
	

	// draw expanding quad fractal
	set_palette(palette0);
	set_color(0);
	clear_screen();
	draw_fade();

	// back go double buffer mode
	toggle_screen();
	set_palette(palette1);
	set_color(0);
	clear_screen();
	swap_screen();
	clear_screen();
	swap_screen();
	
	// draw rotating shapes
	angle = 0;
	set_color(1);
	for (i = 0; i < 2; i++)  // clear slightly larger rectangle background
	{
		fill_rectangle((WIDTH/2) - rx - 16, (HEIGHT/2) - ry - 8,
		(WIDTH/2) + rx + 16, (HEIGHT/2) + ry + 8);
		swap_screen();
	}
	while (angle <= 256)		// rotate through one revolution
	{
		set_color(1);					// clear background rectangle
		fill_rectangle((WIDTH/2) - rx, (HEIGHT/2) - ry,
		(WIDTH/2) + rx, (HEIGHT/2) + ry);
		set_color(2);
		create_shape(5, -angle);       // pentagon
		if (angle >= 200)
		fill_polygon(vertex, 5);
		else
		{
			draw_polygon(vertex, 5);
			draw_fractal(5);
		}
		set_color(3);
		create_shape(3, angle);      // triangle
		if (angle >= 200)
		fill_polygon(vertex, 3);
		else
		{
			draw_polygon(vertex, 3);
			draw_fractal(3);
		}
		set_color(0);
		if (angle < 200)
		{
			print_text(19,11,"The 6502 can");
			print_text(19,12,"fake rotation");
			print_text(19,13,"pretty well.");
		}
		if (angle > 212)
		{
			print_text(8,3,"Respect to");
			print_text(8,4,"the Z80");
			print_text(8,5,"scene.");
		}
		set_color(1);
		angle += 2;		// increment angle by 2 machine degrees
		swap_screen();
	}

	// draw rotating square
	angle = 0;
	set_color(1);
	for (i = 0; i < 2; i++)  // clear slightly larger rectangle background
	{
		fill_rectangle((WIDTH/2) - rx - 16, (HEIGHT/2) - ry - 8,
		(WIDTH/2) + rx + 16, (HEIGHT/2) + ry + 8);
		swap_screen();
	}
	set_palette(palette2);
	while (angle < 256)		// rotate through 1 revolution
	{
		set_color(1);					// clear background rectangle
		rx = 50; ry = 80;
		fill_rectangle((WIDTH/2) - rx, (HEIGHT/2) - ry,
		(WIDTH/2) + rx, (HEIGHT/2) + ry);
		set_color(2);
		create_shape(4, angle);
		fill_polygon(vertex, 4);
		set_color(0);
		draw_polygon(vertex, 4);
		set_color(3);
		rx = 47; ry = 76;
		create_shape(4, angle);
		draw_fractal(4);
		set_color(0);
		print_text(24,11,"Finally a");
		print_text(24,12,"universe");
		print_text(24,13,"in a box.");
		angle += 4;		// increment angle by 2 machine degrees
		swap_screen();
	}
	// draw receding rectangle
	set_palette(palette3);
	i = 0;
	while (++i < 50)
	{
		x = i;
		y = x;
		x1 = (WIDTH-1) - i ;
		y1 = (HEIGHT-1) - i;

		x += 3;
		y += 3;
		x1 -= 3;
		y1 -= 3;
		vertex[0] = x; vertex[1] = y;
		vertex[2] = x; vertex[3] = y1;
		vertex[4] = x1; vertex[5] = y1;
		vertex[6] = x1; vertex[7] = y;
		x -= 3;
		y -= 3;
		x1 += 3;
		y1 += 3;

		set_color(2);
		fill_rectangle(x, y, x1, y1);
		set_color(3);
		draw_fractal(4);
		set_color(0);
		print_text(19-5,6,"I'd like to");
		print_text(19-7,7,"thank Mr. Locke");
		print_text(19-6,8,"for believing");
		print_text(19-3,9,"in me.");
		set_color(1);
		--x; --y; ++x1; ++y1;
		draw_rectangle(x, y, x1, y1);		// fill outside space
		swap_screen();
		draw_rectangle(x, y, x1, y1);		// also in back buffer!
	}
	
	// draw to front buffer (immediately visible)
	toggle_screen();
	
	// erase by quad fractal
	set_color(3);
	full_rect();
	set_color(0);
	i = 0;
	while(--i)
	quad_fractal(0);
	
	// exit graphics!
	set_palette(palette1);
	draw_fade();
	exit_graphics();
	/*
	// get timer end
	t1 = get_time();
	// print out timer data
	// puts("  finished...\n");
	i = *COLPM2 & (7 << 1);		// get the PAL/NTSC flag
	// printf("  timer freq  =          ");
	if (i > 0)
	j = 60;
	else
	j = 50;
	// printf("%d Hz\n", j);
	// printf("  stop        =   %12ld\n", t1);
	// printf("  start       =   %12ld\n", t0);
	// printf("  ticks       =   %12ld\n", t1 - t0);
	// printf("  seconds     =   %12ld\n", (t1 - t0) / j);
	// puts("");
	// puts("  press a key to exit");
*/
	//	wait_for_key();
	*KEYDIS = 0;
	return 0;
}

void full_rect(void)				// make shape for entire screen
{
	vertex[0] = 0; vertex[1] = 0;
	vertex[2] = 0; vertex[3] = HEIGHT;
	vertex[4] = WIDTH; vertex[5] = HEIGHT;
	vertex[6] = WIDTH; vertex[7] = 0;
}

void draw_fade(void)
{
	byte i, x, y, x1, y1;
	// draw expanding quad fractal
	i = 20;
	while(++i < 75)
	{
		set_color(i & 0x3);
		x = (WIDTH/2) - i;
		y = (HEIGHT/2) - i;
		x1 = (WIDTH/2) + i;
		y1 = (HEIGHT/2) + i;
		vertex[0] = x; vertex[1] = y;
		vertex[2] = x; vertex[3] = y1;
		vertex[4] = x1; vertex[5] = y1;
		vertex[6] = x1; vertex[7] = y;
		quad_fractal(0);
	}
}

void draw_fractal(byte sides)
{
	// init plot coords
	*X0 = WIDTH/2;
	*Y0 = HEIGHT/2;
	// use first scanline pointer variable to point at the shape's vertex array
	SC[0] = (uint) vertex;
	// use 2nd scanline pointer var to point at the shape's random vertex indexes
	SC[1] = (uint) &rand_index[sides-3][0];
	// use the GP zero page variable as the interation counter
	*GP = FRACT;
	// call assembler inner loop
	fractal();
}

void quad_fractal(byte interations)
{
	// init plot coords
	*X0 = WIDTH/2;
	*Y0 = HEIGHT/2;
	// use first scanline pointer variable to point at the shape's vertex array
	SC[0] = (uint) vertex;
	// use the GP zero page variable as the interation counter
	*GP = interations;
	// call assembler inner loop
	quad_rand();
}

void init_fractal(byte sides)
{
	byte i;			// indexes
	byte *p;			// pointer to random vertex index array for this shape
	p = &rand_index[sides-3][0];		// -3 == table starts at triangle
	for (i = 0; i < FRACT; i++)
	*p++ = (*RAND % sides) << 1;	// choose random vertex index
}

// create polygon shape at specified angle
void create_shape(byte sides, int angle)
{
	int s, c;		// sine, cosine
	byte i, k;
	int j;
	j = angle;		// starting angle
	i = 0;			// vertex component index
	for (k = 0; k < sides; k++)
	{
		s = (int) ((sin(j) * rx) >> FP_SHIFT);
		c = (int) ((cos(j) * ry) >> FP_SHIFT);
		vertex[i++] = (byte) ((cx+s)>>FP_SHIFT);
		vertex[i++] = (byte) ((cy+c)>>FP_SHIFT);
		j += (512 / sides) + 1;
	}
}
/*
ulong get_time(void)
{
	ulong t;
	byte i;
	time[0] = STIME[0];  // copy current time  from system locations
	time[1] = STIME[1];
	time[2] = STIME[2];
	t = 0;
	for (i = 0; i < 3; i++)
	{
		t <<= 8;
		t |= time[i];
	}
	return t;
}

void wait_for_key(void)
{
	// keyboard status
	byte status;
	while (TRUE)
	{
		// wait for key press
		status = *SKSTAT;                  // read keyboard status register
		if (status & KB_OVERRUN)             // reset keyboard if overrun detected
		*KBREST = TRUE;
		if (!(status & KB_PRESSED))  // if key is being pressed (status bit is zero)
		break; // break out of loop
	}
}
*/