;
; fill_rect.s													10/8/2008
;
;	ANTIC mode E fill rectangle routine.  This routine is similar to the horizontal
; line routine, so you should look at that code first.  Both routines are optimized
; by filling whole bytes instead of per-pixel plotting.  The rectangle routine is
; further optimized by using 16 scanline pointers so that the loop counter increment
; doesn't take up half the loop!
;
; author:
;
;   Bart Jaszcz
;

		.include "graphics.inc"
		.importzp ptr1, tmp1, tmp2, tmp3, tmp4
		.export _fill_rect
		.macpack generic

; 8 bit variables

sf = tmp1							; start of byte fill
ef = tmp3							; end of byte fill
sr = tmp2							; start remainder (pixels)
er = tmp4							; end remainder (pixels)
bc = ptr1							; byte fill count
cy = ptr1+1							; current-y coordinate

.proc _fill_rect

_fill_rect:		lda X1				; is X1 < X0? (not allowed)
				sub X0
				bcs @cont			; no, skip over rts
				rts					; yes, return immediately

@cont:			lda X1				; calc end of byte fill which is just X1 >> 2
				lsr a				; do the shifting
				lsr a				; then
				sta ef				; save the result in ef (end of byte fill)

				lda X1				; calc end remainder pixels (not byte filled)
				and #3				; which is (X1 & 3) ^ 3
				eor #3				; invert bits
				sta er				; save result in er (end remainder)
				
				lda X0				; calc start of byte fill which is (X0 >> 2) + 1
				lsr a				; do the shifting
				lsr a				; then
				sta sf				; save the result in sf (start of byte fill)
				inc sf				; add one at the end
				
				lda X0				; calc start remainder pixels (not byte filled)
				and #3				; which is just X0 & 3
				sta sr				; save result in sr (start remainder)
				
				lda ef				; reload end of byte fill into a
				cmp sf				; compare with start of byte fill
				bpl test_start		; if end >= start goto test start remainder

single_byte:	lda sr				; load start remainder into accumulator
				add #4				; skip over pixel plot masks
				tay					; set offset into mask table
				lda (PM), y			; load mask for start remainder
				sta sr				; save mask in start remainder
				lda er				; load end remainder into accumulator
				add #8				; skip over plot and start remainder masks
				tay					; set offset register
				lda (PM), y			; get end remainder mask
				and sr				; combine with start remainder mask
				sta sr				; save combined mask in the same place
				eor #$ff			; invert mask to clear bits in screen byte
				tax					; save inverted mask in x register
				ldy Y0				; set current-y to Y0
				sty cy

@loop:			lda (SH), y			; load scanline pointer high byte into a 
				sta SC+1			; store it to high byte of scan
				lda (SL), y			; load scanline pointer low byte into a 
				sta SC				; store it to low byte of scan
				ldy ef				; load offset register with end of byte fill index
				txa					; retrive saved inverted mask
				and (SC), y			; and screen byte to be set with inverted mask
				sta GP				; save result in general purpose zero page byte
				lda sr				; retrive saved mask
				and CL				; and it with color filled byte
				ora GP				; or it with saved masked screen byte
				sta (SC), y			; output to screen byte
				inc cy				; increment current-y
				ldy cy				; load current-y coordinate into y register
				cpy Y1				; compare with Y1
				bcc @loop			; loop while cy <= Y1
				beq @loop
				rts					; return (single byte column case)
				
test_start:		lda sr				; load start remainder into a
				bne handle_start	; if not 0 handle start remainder pixels
				dec sf				; otherwise increase start of byte fill
				bpl test_end		; goto test end remainder

handle_start:	add #4				; skip over plot masks in mask table
				tay					; transfer result to offset register
				lda (PM), y			; load start remainder mask
				sta sr				; save it back in start remainder
				eor #$ff			; invert mask
				tax					; save inverted mask in x register
				ldy Y0				; set current-y to Y0
				sty cy

@loop:			lda (SH), y			; load scanline pointer high byte into a 
				sta SC+1			; store it to high byte of scan
				lda (SL), y			; load scanline pointer low byte into a 
				sta SC				; store it to low byte of scan
				ldy sf				; load start of byte fill index into y
				dey					; get index of byte one to the left of this offset
				txa					; retrive saved inverted mask
				and (SC), y			; and the screen byte there with inverted mask
				sta GP				; save result in general purpose zero page byte
				lda sr				; retrive saved mask
				and CL				; and with color byte
				ora GP				; or with masked screen byte
				sta (SC), y			; ouput result to screen
				inc cy				; increment current-y
				ldy cy				; load it into y register
				cpy Y1				; compare with Y1
				bcc @loop			; loop while cy <= Y1
				beq @loop
				
test_end:		lda er				; load end remainder into a
				bne handle_end		; if not 0 handle end remainder pixels
				inc ef				; otherwise increase end of byte fill
				bpl byte_fill		; jump to byte fill

handle_end:		add #8				; skip over plot and start remainder masks
				tay					; transfer result to offset register
				lda (PM), y			; load end remainder mask
				sta er				; save it back in end remainder
				eor #$ff			; invert mask
				tax					; save inverted mask in x register
				ldy Y0				; set current-y to Y0
				sty cy

@loop:			lda (SH), y			; load scanline pointer high byte into a 
				sta SC+1			; store it to high byte of scan
				lda (SL), y			; load scanline pointer low byte into a 
				sta SC				; store it to low byte of scan
				ldy ef				; load end of byte fill index into y
				txa					; retrive saved inverted mask
				and (SC), y			; and the screen byte there with inverted mask
				sta GP				; save result in general purpose zero page byte
				lda er				; retrive saved mask
				and CL				; and with color byte
				ora GP				; or with masked screen byte
				sta (SC), y			; ouput result to screen
				inc cy				; increment current-y
				ldy cy				; load it into y register
				cpy Y1				; compare with Y1
				bcc @loop			; loop while cy <= Y1
				beq @loop

byte_fill:		lda ef				; calculate byte fill count (bc)
				sub sf
				beq return			; skip byte fill code if byte fill count == 0
				bcc return			; also skip byte fill code if byte fill count < 0
				sta bc				; otherwise set byte count
				lda Y0				; set current-y to Y0 
				sta cy

check_y:		lda Y1				; subtract current-y from Y1
				sub cy
				bcs cont			; loop while current-y <= Y1
return:			rts					; otherwise return from routine
				
cont:			ldy cy				; init scanline index
				cmp #16				; was Y1-Y0 >= 16?
				bcc set_1			; no, jump to remainder scanline fill (<16 scanlines)

				ldx #0				; otherwise init counter for 16 pointer optimization
set_16:			lda (SL), y			; load low byte of scanline pointer 
				sta SC, x			; set low byte zero page pointer 
				inx					; point at high byte of zero page pointer
				lda (SH), y			; load high byte of scanline pointer
				sta SC, x			; store high byte of zero page pointer
				inx					; next zero page pointer
				iny					; next scanline pointer
				cpx #32				; done with 16 pointers?
				bne set_16			; no, next interation
				sty cy				; increment current-y by 16 (result was already in y)
				ldy sf				; load byte offset to inside rectangle
				ldx bc				; load byte count to be set (rectangle scanline bytes)
				lda CL				; load current color
loop_16:		sta (SC+0), y		; set 16 bytes inside rectangle on 16 scanlines
				sta (SC+2), y
				sta (SC+4), y
				sta (SC+6), y
				sta (SC+8), y
				sta (SC+10), y
				sta (SC+12), y
				sta (SC+14), y
				sta (SC+16), y
				sta (SC+18), y
				sta (SC+20), y
				sta (SC+22), y
				sta (SC+24), y
				sta (SC+26), y
				sta (SC+28), y
				sta (SC+30), y
				iny					; increment byte offset inside rectangle
				dex					; decrement byte counter
				bne loop_16			; loop until counter is zero
				beq check_y			; go back to Y0<Y1 check

set_1:			lda (SL), y			; fill remainder rectangle scanlines (<16 scanlines)
				sta SC				; using only one zero page scanline pointer
				lda (SH), y
				sta SC+1
				inc cy				; increment current-y
				ldy sf				; load byte offset into rectangle
				ldx bc				; number of bytes to set
				lda CL				; color
loop_1:			sta (SC), y			; set the byte
				iny					; increment byte offset
				dex					; decrement counter
				bne loop_1			; loop until count reached
				beq check_y			; go back to Y0<Y1 check

				
.endproc

