;
; fractal.s
;
;   6502 asm shape fractals.
;
; author:
;
;   Bart Jaszcz
;

	
		.include "graphics.inc"
		.import _xy_plot_0
		.export _fractal, _quad_rand

; random number generator
		
RAND = $d20a

		
; zero page pointers

va = SC					; shape vertex array pointer
vi = SC+2				; random vertex index array pointer

; fractal routine which uses a stored array of random numbers

.proc _fractal

_fractal:			ldy GP				; load interations count into y register (this will serve as offset into vi array)
					lda (vi), y			; load shape random vertex index into accumulator
					tay					; get random vertex index into y register so it can be used as an offset
					lda (va), y			; load x coordinate of random vertex into accumulator
					clc					; clear carry flag
					adc X0				; add current x coordinate
					ror					; divide by 2 (use rotate-right instuction to include carry bit)
					sta X0				; store result back in x coordinate
					iny					; increment array offset (this points at y coordinate)
					lda (va), y			; load y coordinate of vertex into accumulator
					clc					; clear carry flag
					adc Y0				; add current y coordinate
					ror					; divide by 2 (use rotate-right instuction to include carry bit)
					sta Y0				; store result back in y coordinate
					jsr _xy_plot_0		; plot current xy coordinates
					dec GP				; decrement interations count
					bne _fractal		; if not zero, jump to beginning
					rts					; otherwise return
						
.endproc

; fractal routine that uses a new random number every interation (quad only!)

.proc _quad_rand

_quad_rand:			ldx GP				; load interations count into x register
@loop:				lda #3				; set vertex index mask for random number (quad)
					and RAND			; and mask with random number
					asl					; multiply random vertex index by 2 (coordinates per vertex)
					tay					; get random vertex index into y register so it can be used as an offset
					lda (va), y			; load x coordinate of random vertex into accumulator
					clc					; clear carry flag
					adc X0				; add current x coordinate
					ror					; divide by 2 (use rotate-right instuction to include carry bit)
					sta X0				; store result back in x coordinate
					iny					; increment array offset (this points at y coordinate)
					lda (va), y			; load y coordinate of vertex into accumulator
					clc					; clear carry flag
					adc Y0				; add current y coordinate
					ror					; divide by 2 (use rotate-right instuction to include carry bit)
					sta Y0				; store result back in y coordinate
					jsr _xy_plot_0		; plot current xy coordinates
					dex 				; decrement interations count
					bne @loop			; if not zero, jump to beginning
					rts					; otherwise return
						
.endproc



