;
; h_line.s																9/30/2008
;
;	ANTIC mode E horizontal line routine.
;
; author:
;
;   Bart Jaszcz
;

		.include "graphics.inc"
		.importzp tmp1, tmp2, tmp3, tmp4
		.import _xy_plot_0, _xy_plot_1
		.export _h_line
		.macpack generic

; 8 bit variables

sf = tmp1							; start of byte fill
ef = tmp3							; end of byte fill
sr = tmp2							; start remainder (pixels)
er = tmp4							; end remainder (pixels)
		
.proc _h_line

_h_line:		lda X1				; is X1 < X0? (not allowed)
				sub X0
				bcs skip			; no, skip over rts
				rts					; yes, return immediately

skip:			bne cont			; was X1 == X0?  if not, continue to main code
				jsr _xy_plot_0		; yes, just plot one of the endpoints (the first)
				rts					; and return

cont:			lda X1				; calc end of byte fill which is just X1 >> 2
				lsr a				; do the shifting
				lsr a				; then
				sta ef				; save the result in ef (end of byte fill)

				lda X1				; calc end remainder pixels (not byte filled)
				and #3				; which is (X1 & 3) ^ 3
				eor #3				; invert bits
				sta er				; save result in er (end remainder)
				
				lda X0				; calc start of byte fill which is (X0 >> 2) + 1
				lsr a				; do the shifting
				lsr a				; then
				sta sf				; save the result in sf (start of byte fill)
				inc sf				; add one at the end
				
				lda X0				; calc start remainder pixels (not byte filled)
				and #3				; which is just X0 & 3
				sta sr				; save result in sr (start remainder)

				ldy Y0				; load y coordinate into y register
				lda (SH), y			; load scanline pointer high byte into a 
				sta SC+1			; store it to high byte of scan
				lda (SL), y			; load scanline pointer low byte into a 
				sta SC				; store it to low byte of scan
				
				lda ef				; reload end of byte fill into a
				cmp sf				; compare with start of byte fill
				bpl test_start		; if end >= start goto test start remainder

single_byte:	lda sr				; load start remainder into accumulator
				add #4				; skip over pixel plot masks
				tay					; set offset into mask table
				lda (PM), y			; load mask for start remainder
				sta sr				; save mask in start remainder
				lda er				; load end remainder into accumulator
				add #8				; skip over plot and start remainder masks
				tay					; set offset register
				lda (PM), y			; get end remainder mask
				and sr				; combine with start remainder mask
				sta sr				; save combined mask in the same place
				eor #$ff			; invert mask to clear bits in screen byte
				ldy ef				; load offset register with end of byte fill index
				and (SC), y			; and screen byte to be set with inverted mask
				sta er				; save result in end remainder
				lda sr				; retrive saved mask
				and CL				; and it with color filled byte
				ora er				; or it with saved masked screen byte
				sta (SC), y			; output to screen byte
				rts					; return
				
test_start:		lda sr				; load start remainder into a
				bne handle_start	; if not 0 handle start remainder pixels
				dec sf				; otherwise increase start of byte fill
				bpl test_end		; goto test end remainder

handle_start:	add #4				; skip over plot masks in mask table
				tay					; transfer result to offset register
				lda (PM), y			; load start remainder mask
				tax					; save it in x register
				eor #$ff			; invert mask
				ldy sf				; load start of byte fill index into y
				dey					; get index of byte one to the left of this offset
				and (SC), y			; and the screen byte there with inverted mask
				sta sr				; save result in start remainder
				txa					; retrive saved mask
				and CL				; and with color byte
				ora sr				; or with masked screen byte
				sta (SC), y			; ouput result to screen

test_end:		lda er				; load end remainder into a
				bne handle_end		; if not 0 handle end remainder pixels
				inc ef				; otherwise increase end of byte fill
				bpl byte_fill		; jump to byte fill

handle_end:		add #8				; skip over plot and start remainder masks
				tay					; transfer result to offset register
				lda (PM), y			; load end remainder mask
				tax					; save it in x register
				eor #$ff			; invert mask
				ldy ef				; load end of byte fill index into y
				and (SC), y			; and the screen byte there with inverted mask
				sta er				; save result in end remainder
				txa					; retrive saved mask
				and CL				; and with color byte
				ora er				; or with masked screen byte
				sta (SC), y			; ouput result to screen
				
byte_fill:		ldy ef				; load end of byte fill into y
				tya					; make a copy in accumulator
				sub sf				; subtract start of byte fill
				bne do_fill			; return if start equals end
				rts					; which is the two byte case
do_fill:		tax					; transfer result to x register (loop counter)
				lda CL				; load color filled byte into a
loop:			dey					; decrement byte index
				sta (SC), y			; output byte to screen
				dex					; decrement loop counter
				bne loop			; loop until done
				rts					; return when done
.endproc

