;
; math16.inc		9/20/2008
;
; 6502 assembler 16-bit math macros
;
; parameters are pointers to 16 bit ints
; result is always stored in var1
;
; author: Bart Jaszcz
;

; set variable (var1 = var2)
.macro	set16 var1, var2
		lda var2		; load low byte
		sta var1		; set low byte
		lda var2+1		; load high byte
		sta var1+1		; set high byte
.endmacro

; addition (var1 = var1 + var2)
.macro	add16 var1, var2
		clc			; clear carry flag
		lda var1		; load low byte of var1 into a
   	   	adc	var2		; add low byte of var2
   	    sta	var1		; save it in low byte of var1
   	   	lda var1+1		; load high byte of var1
    	adc	var2+1		; add high byte of var2
		sta var1+1		; store result in high byte of var1
.endmacro

; summation (var1 = var2 + var3)
.macro	sum16 var1, var2, var3
		clc			; clear carry flag
		lda var2		; load low byte of var2 into a
     	adc	var3		; add low byte of var3
    	sta	var1		; save it in low byte of var1
    	lda var2+1		; load high byte of var2
    	adc	var3+1		; add high byte of var3
		sta var1+1		; store result in high byte of var1
.endmacro

; subtraction (var1 = var1 - var2)
.macro	sub16 var1, var2
		sec			; set carry flag 
		lda var1		; load low byte of var1
     	sbc	var2		; subtract low byte of var2 
		sta var1		; store result in low byte of var1
		lda var1+1		; load high byte of var1
		sbc	var2+1		; subtract high byte of var2
		sta var1+1		; store result in high byte of var1
.endmacro

; difference (var1 = var2 - var3)
.macro	dif16 var1, var2, var3
		sec			; set carry flag
		lda var2		; load low byte of var2
    	sbc	var3		; subtract low byte of var3
		sta var1		; store result in low byte of var1
		lda var2+1		; load high byte of var2
		sbc	var3+1		; subtract high byte of var3
		sta var1+1		; store result in high byte of var1
.endmacro

; arithmetic shift left (var1 = var1 * 2)
.macro	asl16 var1
		asl var1		; shift low byte and put bit 7 into carry
		rol var1+1		; shift high byte and put carry into bit 0
.endmacro		

; negate (var1 = -var1)
.macro	neg16 var1
		sec			; set carry flag
		lda #0			; negate by subtracting from 0
		sbc var1		; sub low byte of var1
		sta var1		; store in low byte of var1
		lda #0			; negate by subtracting from 0
		sbc var1+1		; sub high byte of var1
		sta var1+1		; store in high byte of var1
.endmacro

