;
; print_text.s																							4/9/2009
;
;    6502 assembler print text to screen in ATARI graphics mode E using a 4x6 font. 
;
; author:
;
;   Bart Jaszcz
;

		.include "graphics.inc"
		.include "math16.inc"
		.importzp ptr1, ptr2, ptr3, ptr4, tmp1, tmp2, tmp3, tmp4
		.import _font
		.export _print_text

; zero page bytes

col = X0				; column on screen (there are 40 columns)
row = Y0				; row on screen (there are 16 rows)

; zero page 16 bit integers

si = SC + 24		; screen scanline increment
fi = SC + 26		; font scanline increment  
ci = SC + 28		; column increment 

; zero page pointers

s0 = SC				; 12 screen scanline pointers (by default the font is doubled in height)
s1 = SC + 2
s2 = SC + 4
s3 = SC + 6
s4 = SC + 8
s5 = SC + 10
s6 = SC + 12
s7 = SC + 14
s8 = SC + 16
s9 = SC + 18
s10 = SC + 20
s11 = SC + 22

f0 = ptr1			; 6 font bitmap scanline pointers (source scanlines)
f1 = ptr2
f2 = ptr3
f3 = ptr4
f4 = tmp1
f5 = tmp3

str = X1			; pointer to null terminated C string to be printed
off = X0			; string character offset variable
chr = Y0			; string character

; first define output character scanlines macro
.macro output font, scan1, scan2
					ldy chr						; load character into y
					lda (font), y				; load font byte for character
					sta GP						; save it in gp var
					eor #$ff					; calc mask
					tax							; save it in x reg
					lda GP						; reload font byte
					and CL						; and with color
					sta GP						; store back in gp
					ldy off						; load string offset into y
					txa							; reload mask
					and (scan1), y				; and screen with mask
					ora GP						; or it with color font byte
					sta (scan1), y				; store it back to screen
					txa							; reload mask
					and (scan2), y				; and screen with mask
					ora GP						; or it with color font byte
					sta (scan2), y				; store it back to screen
.endmacro

; print function starts here

.proc _print_text

_print_text:			lda #0					; init increments
						sta si+1				; clear hi bytes
						sta fi+1
						sta ci+1
						lda #40					; screen scanline increment
						sta si
						lda #96					; font scanline increment
						sta fi
						lda col					; column increment (string x position or column on screen)
						sta ci
						
						lda row					; multiply row (string y position) by 12 (double font height) 
						asl						; 12 == 3 * 4.. so first mult row by 4 then add this result twice
						asl
						sta row					; store result of mult by 4
						clc
						adc row					; add twice
						adc row
						tay						; store result (y coordinate) in y register
						
						lda (SH), y				; load scanline pointer high byte into a 
						sta s0+1				; store it to high byte of screen pointer 0
						lda (SL), y				; load scanline pointer low byte into a 
						sta s0					; store it to low byte of screen pointer 0
						add16 s0, ci			; add column increment to screen pointer 0
						sum16 s1, s0, si		; set the rest of screen pointers
						sum16 s2, s1, si
						sum16 s3, s2, si
						sum16 s4, s3, si
						sum16 s5, s4, si
						sum16 s6, s5, si
						sum16 s7, s6, si
						sum16 s8, s7, si
						sum16 s9, s8, si
						sum16 s10, s9, si
						sum16 s11, s10, si
						
						lda #<_font				; get low byte of font address
						sta f0					; set low byte of font pointer 0
						lda #>_font				; get high byte of font address
						sta f0+1				; set high byte of font pointer 0
						sum16 f1, f0, fi		; set the rest of font pointers
						sum16 f2, f1, fi
						sum16 f3, f2, fi
						sum16 f4, f3, fi
						sum16 f5, f4, fi
						
						lda #0					; init string offset
						sta off
						
@loop:					ldy off					; load character offset into y register
						lda (str), y			; load string character into accumulator
						bne @cont				; continue if character not terminator (null)
						rts						; otherwise return from routine

@cont:					ldx #32					; subtract index of first character in font (32)
						stx GP
						sec
						sbc GP
						sta chr					; save character
						
						output f0, s0, s1		; output character scanlines to screen
						output f1, s2, s3
						output f2, s4, s5
						output f3, s6, s7
						output f4, s8, s9
						output f5, s10, s11
						
						inc off					; next character in string
						jmp @loop				; go back to top of loop
.endproc

