;
; xy_plot.s
;
;	ANTIC mode E plot routine.  Note there are two copies of
; this function with separate coordinates.  The line function
; uses both coordinates to draw symmetricly.
;
; author:
;
;   Bart Jaszcz
;

		.include "graphics.inc"
		.importzp ptr1, tmp1, tmp2
		.export _xy_plot_0
		.export _xy_plot_1
		
; 16 bit variables:

scan = ptr1							; scanline pointer

; 8 bit variables

im = tmp2							; inverted mask
pv = tmp1							; pixel value

.proc _xy_plot_0

_xy_plot_0:		ldy Y0				; load y coordinate into y register
				lda (SH), y			; load scanline pointer high byte into a 
				sta scan+1			; store it to high byte of scan
				lda (SL), y			; load scanline pointer low byte into a 
				sta scan			; store it to low byte of scan

				lda X0				; load x coordinate into a
				and #3				; calc x % 4 (pixel location within a byte)
				tay					; transfer result to y register
				lda (PM), y			; get the pixel mask for this location
				sta pv				; and save it in pv
				eor #$ff			; invert mask bits
				sta im				; and store result in im
				
				lda CL				; get byte filled with drawing color
				and pv				; and it with pixel mask
				sta pv				; then store it back to pv
				
				lda X0				; reload x coordinate into a
				lsr a				; calc x / 4 which is the offset
				lsr a				; to the screen byte to be set
				tay					; transfer result to y register
				lda (scan), y		; get the screen byte into a
				and im				; and it with inverted mask
				ora pv				; then or it with pixel value

				sta (scan), y		; store result in the screen byte
				rts					; return to calling program
.endproc

.proc _xy_plot_1

_xy_plot_1:		ldy Y1				; load y coordinate into y register
				lda (SH), y			; load scanline pointer high byte into a 
				sta scan+1			; store it to high byte of scan
				lda (SL), y			; load scanline pointer low byte into a 
				sta scan			; store it to low byte of scan

				lda X1				; load x coordinate into a
				and #3				; calc x % 4 (pixel location within a byte)
				tay					; transfer result to y register
				lda (PM), y			; get the pixel mask for this location
				sta pv				; and save it in pv
				eor #$ff			; invert mask bits
				sta im				; and store result in im
				
				lda CL				; get byte filled with drawing color
				and pv				; and it with pixel mask
				sta pv				; then store it back to pv
				
				lda X1				; reload x coordinate into a
				lsr a				; calc x / 4 which is the offset
				lsr a				; to the screen byte to be set
				tay					; transfer result to y register
				lda (scan), y		; get the screen byte into a
				and im				; and it with inverted mask
				ora pv				; then or it with pixel value

				sta (scan), y		; store result in the screen byte
				rts					; return to calling program
.endproc

